﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using FRAM.WcfService.Public.Example.BussgodsReference;

namespace FRAM.WcfService.Public.Example
{
	class Program
	{
		public static byte COUNTY_CODE = 53;
		public static string USERNAME = "user@domain.xyx";
		public static string PASSWORD = "password";

		public static string VIEW_PDF_URL = "http://web-test.bussgods.se/internetfraktsedel/pdf/{0}?viewToken={1}";

		public static string DOWNLOAD_PDF_LABEL = "http://web-test.bussgods.se/internetfraktsedel/nedladdning/{0}?viewToken={1}";
		public static string DOWNLOAD_PDF_A4 = "http://web-test.bussgods.se/internetfraktsedel/nedladdning/a4/{0}?viewToken={1}";

		public static FRAMServiceClient _bussgods = new FRAMServiceClient();

		static void Main(string[] args)
		{
			SaveInternetWaybill();

			if (args.Contains("delivery"))
				SaveInternetWaybill(delivery: true);

			if (args.Contains("sverigefraktbox20"))
				SaveInternetWaybill(sverigefraktbox20kg: true);

			if (args.Contains("sverigefraktbox35"))
				SaveInternetWaybill(sverigefraktbox35kg: true);

			if (args.Contains("halfpallet"))
				SaveInternetWaybill(halfPallet: true);

			if (args.Contains("fullpallet"))
				SaveInternetWaybill(fullPallet: true);

			if (args.Contains("getprice"))
				GetPrice();

			if (args.Contains("getstatuses"))
				GetStatuses("BG9018170564");

			/*
				13 = Extradited (Utlämnad)
				14 = Delivered (Utkörd)
			*/
			if (args.Contains("getstatus"))
				GetStatus("BG9018170564", 13);

			Console.WriteLine("Press Any Key...");
			Console.ReadKey();
		}

		/// <summary>
		/// Shipment/Parcel numbers from Bussgods.
		/// </summary>
		static void SaveInternetWaybill(bool cod = false, bool delivery = false, bool priority = false, bool warranty = false,
			/* Sverigefrakt box */
			bool sverigefraktbox20kg = false,
			bool sverigefraktbox35kg = false,
			/* Pallet */
			bool halfPallet = false,
			bool fullPallet = false)
		{
			// Ping
			var ping = _bussgods.Ping();

			Console.WriteLine("Ping?: Result: {0}, Data: {1}", ping.Result, ping.Data);
			Console.WriteLine("");

			// Login
			var user = _bussgods.Login(USERNAME, PASSWORD);

			if (!user.Result)
			{
				Console.WriteLine(user.Message);
				return;
			}

			// AuthTicket
			var authTicket = new AuthTicket
			{
				CountyCode = COUNTY_CODE,
				Token = user.Data.Token,
				Username = USERNAME
			};

			// Cities
			var cities = _bussgods.GetCities(authTicket);

			if (!cities.Result)
			{
				Console.WriteLine(cities.Message);
				return;
			}

			var fromCity = cities.Data.FirstOrDefault(c => c.Name == "SANDVIKEN");
			var toCity = cities.Data.FirstOrDefault(c => c.Name == "JÖNKÖPING");
			//var toCity = cities.Data.FirstOrDefault(c => c.Name == "Finland"); /* For Finland */

			Console.WriteLine("From City: {0} ({1})", fromCity.Name, fromCity.CityCode);
			Console.WriteLine("To City: {0} ({1})", toCity.Name, toCity.CityCode);
			Console.WriteLine("");

			// InternetWaybill
			var internetWaybill = new InternetWaybill
			{
				//AccountNumber = "8412041234",
				//AccountType = "KTO",
				//Bankgiro = "1234567",
				//ClearingNumber = "3300",
				COD = cod,
				//CODReference = "Kalle",
				CODSum = 0,
				Contents = "Stones",
				CustomerNumber = 1,
				Delivery = delivery,
				DeliveryInstructions = "Leave the package on the stairs.",
				FreeText = "Very nice stones.",
				InsurancePriceID = null,
				NumberOfParcels = 1,
				ParcelType = "Övrigt",
				//Plusgiro = "12345678",
				Product = 1,
				ReceiverAddress = "Gatan 1",
				ReceiverCity = "Ankeborg",
				//ReceiverCustomerNumber = 9000,
				ReceiverCityCode = toCity.CityCode,
				ReceiverStationNumber = null,
				ReceiverEmail = "musse@pigg.nu",
				ReceiverName = "Musse Pigg",
				ReceiverPhone = "012121212",
				ReceiverReference = "Musse",
				ReceiverZipCode = "55625",
				SenderCityCode = fromCity.CityCode,
				SenderStationNumber = null,
				SenderAddress = "Gatan 2",
				SenderCity = "Ankeborg",
				SenderEmail = "kalle@anka.nu",
				SenderName = "Kalle Anka",
				SenderPhone = "013131313",
				SenderReference = "Kalle",
				SenderZipCode = "81131",
				TotalWeight = 1,
				TotalVolume = 0.0024f,
			};

			// Sverigefrakt box
			if (sverigefraktbox20kg)
				internetWaybill.ParcelType = "BBOX20in";
			else if (sverigefraktbox35kg)
				internetWaybill.ParcelType = "BBOX35in";
			// Pallet
			else if (halfPallet)
				internetWaybill.ParcelType = "HPll";
			else if (fullPallet)
				internetWaybill.ParcelType = "FPll";

			// Priority/Warranty (additional services)
			if (priority)
				internetWaybill.Product = 2;
			else if (warranty)
				internetWaybill.Product = 3;

			// Parcels
			var parcels = new List<InternetWaybillParcel>();

			for (int packageNumber = 1; packageNumber <= internetWaybill.NumberOfParcels; packageNumber++)
			{
				var internetWaybillParcel = new InternetWaybillParcel
				{
					PackageNumber = packageNumber,
					Height = 10,
					Length = 20,
					Weight = 5,
					Width = 30
				};

				parcels.Add(internetWaybillParcel);
			}

			internetWaybill.InternetWaybillParcels = parcels.ToArray();

			// Stations
			var fromStation = _bussgods.GetSenderStation(authTicket, 81131, (halfPallet || fullPallet), priority, warranty);
			var toStation = _bussgods.GetReceiverStation(authTicket, 55625, internetWaybill.ParcelType, internetWaybill.InternetWaybillParcels, cod, delivery, (halfPallet || fullPallet), priority, warranty);
			//var toStation = _bussgods.GetStation(authTicket, 5301); /* For Finland */

			if (!fromStation.Result)
			{
				Console.WriteLine(fromStation.Message);
				return;
			}
			if (!toStation.Result)
			{
				Console.WriteLine(toStation.Message);
				return;
			}

			Console.WriteLine("From Station: {0} ({1} / City: {2} ({3})", fromStation.Data.Name, fromStation.Data.StationNumber, fromStation.Data.CityName, fromStation.Data.CityCode);
			Console.WriteLine("To Station: {0} ({1}) / City: {2} ({3})", toStation.Data.Name, toStation.Data.StationNumber, toStation.Data.CityName, toStation.Data.CityCode);
			Console.WriteLine("");

			internetWaybill.SenderStationNumber = fromStation.Data.StationNumber;
			internetWaybill.ReceiverStationNumber = toStation.Data.StationNumber;

			// Result
			var result = _bussgods.SaveInternetWaybill(authTicket, internetWaybill);

			if (!result.Result)
			{
				Console.WriteLine(result.Message);
				return;
			}

			string viewPdfUrl = string.Format(VIEW_PDF_URL, result.Data.ShipmentNumber, result.Data.ViewToken);
			string downloadPdfLabelUrl = string.Format(DOWNLOAD_PDF_LABEL, result.Data.ShipmentNumber, result.Data.ViewToken);
			string downloadPdfA4Url = string.Format(DOWNLOAD_PDF_A4, result.Data.ShipmentNumber, result.Data.ViewToken);

			Console.WriteLine("ShipmentNumber: {0}, View: {1}", result.Data.ShipmentNumber, viewPdfUrl);
			Console.WriteLine("ShipmentNumber: {0}, Download Label (PDF): {1}", result.Data.ShipmentNumber, downloadPdfLabelUrl);
			Console.WriteLine("ShipmentNumber: {0}, Download A4 (PDF): {1}", result.Data.ShipmentNumber, downloadPdfA4Url);

			foreach (var parcel in result.Data.InternetWaybillParcels)
			{
				Console.WriteLine("PackageNumber: {0}, ParcelNumber: {1}", parcel.PackageNumber, parcel.ParcelNumber);
			}

			Console.WriteLine("");
		}

		/// <summary>
		/// Calculates price for Shipment.
		/// </summary>
		static void GetPrice()
		{
			// Ping
			var ping = _bussgods.Ping();

			Console.WriteLine("Ping?: Result: {0}, Data: {1}", ping.Result, ping.Data);
			Console.WriteLine("");

			// Login
			var user = _bussgods.Login(USERNAME, PASSWORD);

			if (!user.Result)
			{
				Console.WriteLine(user.Message);
				return;
			}

			// AuthTicket
			var authTicket = new AuthTicket
			{
				CountyCode = COUNTY_CODE,
				Token = user.Data.Token,
				Username = USERNAME
			};

			// Cities
			var cities = _bussgods.GetCities(authTicket);

			if (!cities.Result)
			{
				Console.WriteLine(cities.Message);
				return;
			}

			var fromCity = cities.Data.FirstOrDefault(c => c.Name == "SANDVIKEN");
			var toCity = cities.Data.FirstOrDefault(c => c.Name == "JÖNKÖPING");
			//var toCity = cities.Data.FirstOrDefault(c => c.Name == "Finland"); /* For Finland */

			Console.WriteLine("From City: {0} ({1})", fromCity.Name, fromCity.CityCode);
			Console.WriteLine("To City: {0} ({1})", toCity.Name, toCity.CityCode);
			Console.WriteLine("");

			// Price Request
			var priceRequest = new PriceRequest
			{
				COD = false,
				CODSum = 0,
				CustomerCountyCode = COUNTY_CODE,
				CustomerNumber = 1,
				Delivery = false,
				InsurancePriceID = null,
				ParcelType = "Övrigt",
				Product = 1,
				ReceiverCustomerCountyCode = null,
				ReceiverCustomerNumber = null,
				ReceiverCityCode = toCity.CityCode,
				SenderCityCode = fromCity.CityCode,
				Weight = 10,
				Volume = 0.10f
			};

			// Result
			var result = _bussgods.GetPrice(authTicket, priceRequest);

			if (!result.Result)
			{
				Console.WriteLine(result.Message);
				return;
			}

			Console.WriteLine("{0} -> {1} ({2} kg, {3} m³) @ {4:C} ({5:C} without VAT)", fromCity.Name, toCity.Name, priceRequest.Weight, priceRequest.Volume, result.Data.TotalFeeWithVAT, result.Data.TotalFee);
			Console.WriteLine("");
		}

		/// <summary>
		/// Statuses.
		/// </summary>
		static void GetStatuses(string shipmentNumber)
		{
			// Ping
			var ping = _bussgods.Ping();

			Console.WriteLine("Ping?: Result: {0}, Data: {1}", ping.Result, ping.Data);
			Console.WriteLine("");

			// Login
			var user = _bussgods.Login(USERNAME, PASSWORD);

			if (!user.Result)
			{
				Console.WriteLine(user.Message);
				return;
			}

			// AuthTicket
			var authTicket = new AuthTicket
			{
				CountyCode = COUNTY_CODE,
				Token = user.Data.Token,
				Username = USERNAME
			};

			// Statuses
			var statuses = _bussgods.GetStatuses(authTicket, shipmentNumber);

			if (!statuses.Result)
			{
				Console.WriteLine(statuses.Message);
				return;
			}

			foreach (var s in statuses.Data)
			{
				Console.WriteLine("[{0}] City: {1}, Status: {2} ({3}), Signature: {4}", s.Timestamp, s.City, s.StatusDescription, s.StatusMember, (s.Signature != null));

				// Signature
				if ((s.StatusMember == 13 /* Extradited (Utlämnad) */ || s.StatusMember == 14 /* Delivered (Utkörd) */)
					&& (s.Signature != null))
				{
					string signatureFilePath = string.Format("C:\\tmp\\signature_{0}_{1}.png", s.StatusMember, s.Signature.Signee);

					File.WriteAllBytes(signatureFilePath, s.Signature.Image);

					Console.WriteLine("Saved Signature: {0}", signatureFilePath);
				}
			}
		}

		/// <summary>
		/// Statuses.
		/// </summary>
		static void GetStatus(string shipmentNumber, short status)
		{
			// Ping
			var ping = _bussgods.Ping();

			Console.WriteLine("Ping?: Result: {0}, Data: {1}", ping.Result, ping.Data);
			Console.WriteLine("");

			// Login
			var user = _bussgods.Login(USERNAME, PASSWORD);

			if (!user.Result)
			{
				Console.WriteLine(user.Message);
				return;
			}

			// AuthTicket
			var authTicket = new AuthTicket
			{
				CountyCode = COUNTY_CODE,
				Token = user.Data.Token,
				Username = USERNAME
			};

			// Statuses
			var statuses = _bussgods.GetStatus(authTicket, shipmentNumber, status);

			if (!statuses.Result)
			{
				Console.WriteLine(statuses.Message);
				return;
			}

			foreach (var s in statuses.Data)
			{
				Console.WriteLine("[{0}] City: {1}, Status: {2} ({3}), Signature: {4}", s.Timestamp, s.City, s.StatusDescription, s.StatusMember, (s.Signature != null));

				// Signature
				if (s.Signature != null)
				{
					string signatureFilePath = string.Format("C:\\tmp\\signature_{0}_{1}.png", s.StatusMember, s.Signature.Signee);

					File.WriteAllBytes(signatureFilePath, s.Signature.Image);

					Console.WriteLine("Saved Signature: {0}", signatureFilePath);
				}
			}
		}
	}
}
